@@ -1,8 +1,8 @@
 
 /*
- * $Id: database.c,v 1.9 2001/06/11 08:40:38 jheinonen Exp $
+ * $Id: database.c,v 1.13 2001/10/12 09:31:44 jheinonen Exp $
  *
- * by JH <jheinonen@bigfoot.com>
+ * by JH <jheinonen@users.sourceforge.net>
  *
  * Copyright (C) Jaakko Heinonen
  */
@@ -58,6 +58,7 @@ struct abook_field abook_fields[ITEM_FIELDS] = {
 	{"Name",	"name",		TAB_CONTACT},/* NAME */
 	{"E-mails",	"email",	TAB_CONTACT},/* EMAIL */
 	{"Address",	"address",	TAB_ADDRESS},/* ADDRESS */
+	{"Address2",	"address2",	TAB_ADDRESS},/* ADDRESS2 */   
 	{"City",	"city",		TAB_ADDRESS},/* CITY */
 	{"State/Province","state",	TAB_ADDRESS},/* STATE */
 	{"ZIP/Postal Code","zip",	TAB_ADDRESS},/* ZIP */
@@ -107,9 +108,12 @@ parse_database(FILE *in)
 		} else if((tmp = strchr(line, '=') ) && sec ) {
 			*tmp++ = '\0';
 			for(i=0; i<ITEM_FIELDS; i++)
-				if( !strcmp(abook_fields[i].key, line) )
+				if( !strcmp(abook_fields[i].key, line) ) {
 					item[i] = strdup(tmp);
+					goto next;
+				}
 		}
+next:
 		free(line);
 	}
 
@@ -144,15 +148,17 @@ write_database(FILE *out, struct db_enumerator e)
 	int j;
 	int i = 0;
 
-	fprintf(out, "# abook addressbook file\n\n");
-	fprintf(out, "[format]\n");
-	fprintf(out, "program=" PACKAGE "\n");
-	fprintf(out, "version=" VERSION "\n");
-	fprintf(out, "\n\n");
+	fprintf(out,
+		"# abook addressbook file\n\n"
+		"[format]\n"
+		"program=" PACKAGE "\n"
+		"version=" VERSION "\n"
+		"\n\n"
+	);
 
 	db_enumerate_items(e) {
 		fprintf(out, "[%d]\n", i);
-		for(j=0; j<ITEM_FIELDS; j++) {
+		for(j = 0; j < ITEM_FIELDS; j++) {
 			if( database[e.item][j] != NULL &&
 					*database[e.item][j] )
 				fprintf(out, "%s=%s\n",
@@ -226,7 +232,7 @@ close_database()
 
 #define _MAX_FIELD_LEN(X)	(X == EMAIL ? MAX_EMAILSTR_LEN:MAX_FIELD_LEN)
 
-inline static void
+static void
 validate_item(list_item item)
 {
 	int i;
