@@ -1,8 +1,8 @@
 
 /*
- * $Id: misc.c,v 1.3 2001/06/11 12:51:54 jheinonen Exp $
+ * $Id: misc.c,v 1.5 2001/08/23 09:39:56 jheinonen Exp $
  *
- * by JH <jheinonen@bigfoot.com>
+ * by JH <jheinonen@users.sourceforge.net>
  *
  * Copyright (C) Jaakko Heinonen
  */
@@ -232,7 +232,7 @@ getaline(FILE *f)
 
 	len = 0;
 	size = thres;
-	buf = abook_malloc(size);
+	buf = (char *)abook_malloc(size);
 
 	while (fgets(buf+len, size-len, f) != NULL) {
 		len += strlen(buf+len);
